
#pDebug #pDebugToolbarHandle {
	position:fixed;
	border:1px solid #bc2131;
	top: 25%;
	right:0;
	z-index:100000000;
}


#pDebug a#pShowToolBarButton {
	display:block;
	height:40px;
	width:40px;
	border-right:none;
	border-bottom:1px solid #fff;
	border-top:1px solid #fff;
	border-left:1px solid #fff;
	color:#fff;
	font-size:10px;
	font-weight:bold;
	text-decoration:none;
	text-align:center;
	text-indent:-999999px;
	background:#bc2131 url(img/pdtb.png) no-repeat left center;
	opacity:0.8;
	filter:alpha(opacity=80);
}

#pDebug a#pShowToolBarButton:hover {
	background-color:#bc2131;
	padding-right:2px;
	border-top-color:#fff;
	border-left-color:#fff;
	border-bottom-color:#fff;
	opacity:1.0;
}

